Package: permuteP
Type: Package
Title: permutation test
Version: 1.4
Date: 2010-07-04
Author: Wei Sun
Maintainer: Wei Sun <weisun@email.unc.edu>
Description: permutation t-test and a permutation test for collapsed rare variantes in genetic assocations studies. 
License: GPL version 2 or later
Packaged: 2010-09-18 18:48:04 UTC; wsun
