\name{pprv}
\alias{pprv}
\title{
  permutation p-value for collapsed rare variants
}
\description{
  permutation p-value for collapsed rare variants, in case-control association studies. 
}
\usage{
pprv(y, x, alpha = NULL)
}
\arguments{
  \item{y}{
  indicator of case-control status
}
  \item{x}{
  a binary data matrix. Each row correponding to one sample and each column corresponding to one SNP. The SNP genotype are coded as binary variable to indicate with or without the rare variant. 
}
  \item{alpha}{
  nominal p-value cutoff for which we calculate permutation p-value. If it is NULL, the minimum nominal p-value across all possible collapsed SNPs is used as alpha. 
}
}
\value{
A list with two components
  \item{p }{ the nominal p-value cutoff alpha}
  \item{perP }{the permutation p-value}
}

\author{ Wei Sun wsun@bios.unc.edu }
\keyword{ methods }
