\name{prv}
\alias{prv}
\title{
  nominal p-value for collapsed rare variants
}
\description{
  nominal p-value for collapsed rare variants
}
\usage{
prv(y, x)
}
\arguments{
  \item{y}{
  indicator of case-control status
}
  \item{x}{
  a binary data matrix. Each row correponding to one sample and each column corresponding to one SNP. The SNP genotype are coded as binary variable to indicate with or without the rare variant. 
}
}
\value{
A list with two components
  \item{comp1 }{p, which equals to alpha}
  \item{comp2 }{perP, the permutation p-value}
}

\author{ Wei Sun wsun@bios.unc.edu }
\keyword{ methods }
